"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
Object.defineProperties(Array.prototype, {
    first: {
        get: function () {
            if (this.length > 0)
                return this[0];
            else
                return undefined;
        }
    },
    last: {
        get: function () {
            if (this.length > 0)
                return this[this.length - 1];
            else
                return undefined;
        }
    },
    isEmpty: {
        get: function () {
            return this.length < 1;
        }
    },
    randomElement: {
        get: function () {
            var _this = this;
            return function () {
                if (_this.isEmpty)
                    return undefined;
                var index = Math.floor(Math.random() * _this.length);
                return _this[index];
            };
        }
    },
    insert: {
        get: function () {
            var _this = this;
            return function (element, at) {
                _this.splice(at, 0, element);
            };
        }
    },
    remove: {
        get: function () {
            var _this = this;
            return function (at) {
                _this.splice(at, 1);
            };
        }
    },
    compactMap: {
        get: function () {
            var _this = this;
            return function (callback) {
                var results = [];
                for (var i = 0; i < _this.length; i++) {
                    var result = callback(_this[i], i, _this);
                    if (result !== undefined && result !== null)
                        results.push(result);
                }
                return results;
            };
        }
    },
    shuffled: {
        get: function () {
            var _this = this;
            return function () {
                var clone = _this.slice();
                clone.shuffle();
                return clone;
            };
        }
    },
    shuffle: {
        get: function () {
            var _this = this;
            return function () {
                var _a;
                for (var i = (_a = _this.endIndex) !== null && _a !== void 0 ? _a : -1; i > 0; i--) {
                    var j = Math.floor(Math.random() * (i + 1));
                    var tmp = _this[i];
                    _this[i] = _this[j];
                    _this[j] = tmp;
                }
            };
        }
    },
    swapAt: {
        get: function () {
            var _this = this;
            return function (indexA, indexB) {
                var temp = _this[indexB];
                _this[indexB] = _this[indexA];
                _this[indexA] = temp;
            };
        }
    },
    startIndex: {
        get: function () {
            var indices = Object.keys(this);
            if (indices.isEmpty || typeof indices.first !== 'string')
                return undefined;
            var startIndex = parseInt(indices.first);
            if (isNaN(startIndex))
                return undefined;
            else
                return startIndex;
        }
    },
    endIndex: {
        get: function () {
            var indices = Object.keys(this);
            if (indices.isEmpty || typeof indices.last !== 'string')
                return undefined;
            var endIndex = parseInt(indices.last);
            if (isNaN(endIndex))
                return undefined;
            else
                return endIndex;
        }
    }
});
//# sourceMappingURL=Array.js.map