"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
Object.defineProperties(String.prototype, {
    isEmpty: {
        get: function () {
            return this.valueOf().length === 0;
        }
    },
    inserted: {
        get: function () {
            var _this = this;
            return function (substring, at) {
                return _this.slice(0, at) + substring + _this.slice(at);
            };
        }
    },
    first: {
        get: function () {
            if (this.length > 0)
                return this[0];
            else
                return undefined;
        }
    },
    last: {
        get: function () {
            if (this.length > 0)
                return this[this.length - 1];
            else
                return undefined;
        }
    },
    randomElement: {
        get: function () {
            var _this = this;
            return function () {
                if (_this.length < 1)
                    return undefined;
                var index = Math.floor(Math.random() * _this.length);
                return _this[index];
            };
        }
    },
    map: {
        get: function () {
            var _this = this;
            return function (callback) {
                var mappedString = '';
                for (var i = 0; i < _this.length; i++) {
                    mappedString += callback(_this[i], i, _this.valueOf());
                }
                return mappedString;
            };
        }
    },
    compactMap: {
        get: function () {
            var _this = this;
            return function (callback) {
                var mappedString = '';
                for (var i = 0; i < _this.length; i++) {
                    var mappedChar = callback(_this[i], i, _this.valueOf());
                    if (typeof mappedChar === 'string')
                        mappedString += mappedChar;
                }
                return mappedString;
            };
        }
    },
    forEach: {
        get: function () {
            var _this = this;
            return function (callback) {
                for (var i = 0; i < _this.length; i++) {
                    callback(_this[i], i, _this.valueOf());
                }
            };
        }
    },
    reduce: {
        get: function () {
            var _this = this;
            return function (callback, initialValue) {
                var val = initialValue;
                for (var i = 0; i < _this.length; i++) {
                    val = callback(val, _this[i], i);
                }
                return val;
            };
        }
    },
    startIndex: {
        get: function () {
            if (this.length > 0)
                return 0;
            else
                return undefined;
        }
    },
    endIndex: {
        get: function () {
            if (this.length > 0)
                return this.length - 1;
            else
                return undefined;
        }
    },
    sorted: {
        get: function () {
            var _this = this;
            return function () {
                return _this.split('').sort().join('');
            };
        }
    },
    reversed: {
        get: function () {
            var _this = this;
            return function () {
                var _a;
                var reversed = '';
                for (var i = (_a = _this.endIndex) !== null && _a !== void 0 ? _a : -1; i >= 0; i--) {
                    reversed += _this[i];
                }
                return reversed;
            };
        }
    },
    shuffled: {
        get: function () {
            var _this = this;
            return function () {
                return _this.split('').shuffled().join('');
            };
        }
    },
    prefix: {
        get: function () {
            var _this = this;
            return function (callback) {
                var prefix = '';
                for (var i = 0; i < _this.length; i++) {
                    var charPassesCallback = callback(_this[i], i, _this.valueOf());
                    if (charPassesCallback)
                        prefix += _this[i];
                    else
                        break;
                }
                return prefix;
            };
        }
    }
});
//# sourceMappingURL=String.js.map